# -*- coding: utf-8 -*-


import sys, os

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [ 'sphinx.ext.intersphinx' ]

# Add any paths that contain templates here, relative to this directory.
templates_path  = ['_templates']
master_doc      = 'index'        # The master toctree document.
source_suffix   = '.rst'         # The suffix of source filenames.
#source_encoding = 'utf-8-sig'    # The encoding of source files.

# General information about the project.
project = 'oncilla-sim'
release = '0.2.0~rc2'              # The full version
version = '0.2'                    # The short X.Y version.
copyright = u'2012, Arne Nordmann, Alexandre Tuleu'
language  = 'en'                        # Language for autogenerated
                                        # content

rst_prolog = """
.. |project| replace::
   Oncilla-Sim

.. |repository| replace::
   https://redmine.amarsi-project.eu/git/oncillasim.git

.. _`AMARSi`:
	https://amarsi-project.eu

.. _`rsc`:
	http://code.cor-lab.org/svn/rsc

.. _`rsb`:
	http://docs.cor-lab.org/rsb-manual

.. _`rst`:
	http://docs.cor-lab.org/rst-manual

.. _spread: http://www.spread.org

.. _cmake: http://www.cmake.org/

.. _google protocol buffers: http://code.google.com/p/protobuf/

.. _Doxygen: http://www.doxygen.nl/
.. _Lcov: http://ltp.sourceforge.net/coverage/lcov.php
.. _cppcheck: http://sourceforge.net/apps/mediawiki/cppcheck/index.php?title=Main_Page

.. _boost: http://www.boost.org/
.. _boost.uuid: http://www.boost.org/doc/libs/1_42_0/libs/uuid/uuid.html

.. |ubuntu| replace::
   Ubuntu GNU/Linux
"""

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

html_context = {
                'doc_cca': os.environ.get('cca', "/"),
                'doc_rsb': os.environ.get('rsb', "/"),
                'doc_rsbag': os.environ.get('rsbag', "/"),
                'doc_rst': os.environ.get('rst', "/"),
                'doc_liboncilla': os.environ.get('liboncilla', "/"),
                'doc_ccaoncilla': os.environ.get('ccaoncilla', "/"),
                }

html_theme = 'corlab'
#html_theme_options = {}
html_theme_path = ['.']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    '**': [ 'globaltoc.html', 'links.html', 'sourcelink.html', 'searchbox.html' ]
}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'oncilla-sim-manual'


# -- Options for LaTeX output --------------------------------------------------

latex_elements = {
    'papersize': 'a4paper',
    'pointsize': '10pt'
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    ('index', 'oncilla-sim-manual.tex', u'Oncilla-Sim Manual', u'Arne Nordmann, Alexandre Tuleu', 'manual'),
    ]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('info',   'info',   u'info Program',   [u'Jan Moringen'], 1),
    ('logger', 'logger', u'logger Program', [u'Jan Moringen'], 1),
    ('send',   'send',   u'send Progam',    [u'Jan Moringen'], 1),
    ('call',   'call',   u'call Progam',    [u'Jan Moringen'], 1)
    ]

# -- Options for intersphinx ---------------------------------------------------

intersphinx_mapping = {
    'python': ('http://docs.python.org/',                      None),
    }

external_modules_list=os.environ['sphinx_external_modules']
if  len(external_modules_list) > 0 :

    for m in os.environ['sphinx_external_modules'].split(':'):
        if os.environ.has_key(m):
            intersphinx_mapping[m] = (os.environ[m], None)
    


